
// Demo program to test out the IR corner sensors

// This program tests for an obstacle with each corner sensor. If an obstacle is 
// found then the corner LED should go from flickering to on solidly. This program
// also continually prints out the amount of IR light detected by the sensors
// in the order 

//           Front Left, Front Right, Rear Left, Rear Right

const int FRONT_LEFT_LED_PIN = 8;
const int FRONT_RIGHT_LED_PIN = 7;
const int REAR_LEFT_LED_PIN = 12;
const int REAR_RIGHT_LED_PIN = 4;

const int FRONT_LEFT_SENSE_PIN = A2;
const int FRONT_RIGHT_SENSE_PIN = A3;
const int REAR_LEFT_SENSE_PIN = A1;
const int REAR_RIGHT_SENSE_PIN = A0;

const int OBSTACLE_DETECTED_IR_LEVEL = 100;

//------------------------------------------------------------------------------
void setup()
{
    Serial.begin( 9600 );
}

//------------------------------------------------------------------------------
void loop()
{
    // Turn on the IR  LEDs
    digitalWrite( FRONT_LEFT_LED_PIN, 1 );
    digitalWrite( FRONT_RIGHT_LED_PIN, 1 );
    digitalWrite( REAR_LEFT_LED_PIN, 1 );
    digitalWrite( REAR_RIGHT_LED_PIN, 1 );
    
    delay( 2 );    // Give sensors time to respond
    
    // Read total IR values
    int frontLeftIR = analogRead( FRONT_LEFT_SENSE_PIN );
    int frontRightIR = analogRead( FRONT_RIGHT_SENSE_PIN );
    int rearLeftIR = analogRead( REAR_LEFT_SENSE_PIN );
    int rearRightIR = analogRead( REAR_RIGHT_SENSE_PIN );
  
    // Turn off the IR LEDs
    digitalWrite( FRONT_LEFT_LED_PIN, 0 );
    digitalWrite( FRONT_RIGHT_LED_PIN, 0 );
    digitalWrite( REAR_LEFT_LED_PIN, 0 );
    digitalWrite( REAR_RIGHT_LED_PIN, 0 );

    delay( 2 );    // Give sensors time to respond

    // Subtract ambient IR from total IR to give reflected IR values
    frontLeftIR -= analogRead( FRONT_LEFT_SENSE_PIN );
    frontRightIR -= analogRead( FRONT_RIGHT_SENSE_PIN );
    rearLeftIR -= analogRead( REAR_LEFT_SENSE_PIN );
    rearRightIR -= analogRead( REAR_RIGHT_SENSE_PIN );
    
    // If an obstacle is close, then the IR value will be above a certain threshold
    digitalWrite( FRONT_LEFT_LED_PIN, frontLeftIR > OBSTACLE_DETECTED_IR_LEVEL );
    digitalWrite( FRONT_RIGHT_LED_PIN, frontRightIR > OBSTACLE_DETECTED_IR_LEVEL );
    digitalWrite( REAR_LEFT_LED_PIN, rearLeftIR > OBSTACLE_DETECTED_IR_LEVEL );
    digitalWrite( REAR_RIGHT_LED_PIN, rearRightIR > OBSTACLE_DETECTED_IR_LEVEL );
    
    // Send the IR levels out over serial
    Serial.print( frontLeftIR );
    Serial.print( ", " );
    Serial.print( frontRightIR );
    Serial.print( ", " );
    Serial.print( rearLeftIR );
    Serial.print( ", " );
    Serial.println( rearRightIR );
    
    delay( 100 );
}


