
// Demo program to test the motors

// This program should make the robot drive forward, spin left, drive backwards,
// and then spin right.

// The motors are modified servo motors so they need to be sent a PWM signal with
// different pulse widths in microseconds (US) in order to drive them.

#include <Servo.h>

//------------------------------------------------------------------------------
const int LEFT_MOTOR_PIN = 9;
const int RIGHT_MOTOR_PIN = 10;
const int MIN_MOTOR_SPEED = -100;
const int MAX_MOTOR_SPEED = 100;

const int LEFT_MOTOR_FULL_REVERSE_US = 2000;
const int LEFT_MOTOR_FULL_STOP_US = 1500;
const int LEFT_MOTOR_FULL_FORWARD_US = 1000;
const int RIGHT_MOTOR_FULL_REVERSE_US = 1000;
const int RIGHT_MOTOR_FULL_STOP_US = 1500;
const int RIGHT_MOTOR_FULL_FORWARD_US = 2000;

Servo gLeftMotor;
Servo gRightMotor;

//------------------------------------------------------------------------------
int convertLeftMotorSpeedToUS( int motorSpeed );
int convertRightMotorSpeedToUS( int motorSpeed );
void driveForwards();
void spinLeft();
void driveBackwards();
void spinRight();
void pause();

//------------------------------------------------------------------------------
void setup()
{
    gLeftMotor.attach( LEFT_MOTOR_PIN );
    gRightMotor.attach( RIGHT_MOTOR_PIN );
}

//------------------------------------------------------------------------------
void loop()
{
    pause(); 
    driveForwards();
    spinLeft();
    driveBackwards();
    spinRight();
    pause();   
}

//------------------------------------------------------------------------------
int convertLeftMotorSpeedToUS( int motorSpeed )
{
    int pulseWidthUS = LEFT_MOTOR_FULL_STOP_US;    // Default result is full stop
    
    motorSpeed = constrain( motorSpeed, MIN_MOTOR_SPEED, MAX_MOTOR_SPEED );
    
    if ( motorSpeed < 0 )
    {
        float proportion = (float)motorSpeed/(float)MIN_MOTOR_SPEED;
        pulseWidthUS = LEFT_MOTOR_FULL_STOP_US 
            + proportion*(LEFT_MOTOR_FULL_REVERSE_US - LEFT_MOTOR_FULL_STOP_US);
    }
    else if ( motorSpeed > 0 )
    {
        float proportion = (float)motorSpeed/(float)MAX_MOTOR_SPEED;
        pulseWidthUS = LEFT_MOTOR_FULL_STOP_US 
            + proportion*(LEFT_MOTOR_FULL_FORWARD_US - LEFT_MOTOR_FULL_STOP_US);
    }
    
    return pulseWidthUS;
}

//------------------------------------------------------------------------------
int convertRightMotorSpeedToUS( int motorSpeed )
{
    int pulseWidthUS = RIGHT_MOTOR_FULL_STOP_US;    // Default result is full stop
    
    motorSpeed = constrain( motorSpeed, MIN_MOTOR_SPEED, MAX_MOTOR_SPEED );
    
    if ( motorSpeed < 0 )
    {
        float proportion = (float)motorSpeed/(float)MIN_MOTOR_SPEED;
        pulseWidthUS = RIGHT_MOTOR_FULL_STOP_US 
            + proportion*(RIGHT_MOTOR_FULL_REVERSE_US - RIGHT_MOTOR_FULL_STOP_US);
    }
    else if ( motorSpeed > 0 )
    {
        float proportion = (float)motorSpeed/(float)MAX_MOTOR_SPEED;
        pulseWidthUS = RIGHT_MOTOR_FULL_STOP_US 
            + proportion*(RIGHT_MOTOR_FULL_FORWARD_US - RIGHT_MOTOR_FULL_STOP_US);
    }
    
    return pulseWidthUS;
}

//------------------------------------------------------------------------------
void driveForwards()
{
    gLeftMotor.writeMicroseconds( convertLeftMotorSpeedToUS( 100 ) );
    gRightMotor.writeMicroseconds( convertRightMotorSpeedToUS( 100 ) );
    delay( 2000 );
}

//------------------------------------------------------------------------------
void spinLeft()
{
    gLeftMotor.writeMicroseconds( convertLeftMotorSpeedToUS( -100 ) );
    gRightMotor.writeMicroseconds( convertRightMotorSpeedToUS( 100 ) );
    delay( 2200 );
}

//------------------------------------------------------------------------------
void driveBackwards()
{
    gLeftMotor.writeMicroseconds( convertLeftMotorSpeedToUS( -100 ) );
    gRightMotor.writeMicroseconds( convertRightMotorSpeedToUS( -100 ) );
    delay( 2000 );
}

//------------------------------------------------------------------------------
void spinRight()
{
    gLeftMotor.writeMicroseconds( convertLeftMotorSpeedToUS( 50 ) );
    gRightMotor.writeMicroseconds( convertRightMotorSpeedToUS( -50 ) );
    delay( 4000 );
}

//------------------------------------------------------------------------------
void pause()
{
    gLeftMotor.writeMicroseconds( convertLeftMotorSpeedToUS( 0 ) );
    gRightMotor.writeMicroseconds( convertRightMotorSpeedToUS( 0 ) );
    delay( 1000 );
}
